package cn.fw.morax.service.biz.calculator.eval.reward;

import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.db.eval.EvalGroupReward;
import cn.fw.morax.domain.db.eval.EvalGroupRewardLadders;
import cn.fw.morax.domain.db.eval.EvalGroupRewardParam;
import cn.fw.morax.domain.db.eval.EvalGroupRewardPrecondition;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.ParamTypeEnum;
import cn.fw.morax.domain.enums.RankTypeEnum;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundZSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author jiangchao
 * @des: 排名正负激励
 * @date 2023/2/1 11:03
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class EvalRewardRankCalculator extends EvalRewardBaseCalculator {

    private final StringRedisTemplate stringRedisTemplate;

    @Value("${spring.cache.custom.global-prefix}:eval:reward:rank")
    @Getter
    private String rank;

    @Override
    public EvalRewardCalMethodEnum getCalMethod() {
        return EvalRewardCalMethodEnum.RANK;
    }

    @Override
    public BigDecimal calculate(EvalGroupUserShop param, EvalGroupReward rewardBO) {
        return null;
    }

    @Override
    public void calculateGroup(List<EvalGroupUserShop> userShops, EvalGroupReward reward) {
        if (PublicUtil.isEmpty(userShops)) {
            return;
        }
        initReward(userShops);
        final Long evalGroupRewardId = reward.getId();
        final String key = getRank() + evalGroupRewardId;
        RankTypeEnum rankType = reward.getRankType();
        List<EvalGroupRewardPrecondition> preconditions = queryPrecondition(evalGroupRewardId);
        List<EvalGroupRewardParam> params = queryProjectParam(evalGroupRewardId, ParamTypeEnum.LADDER);
        if (CollectionUtils.isEmpty(params)) {
            return;
        }

        BoundZSetOperations<String, String> zSetOps = stringRedisTemplate.boundZSetOps(key);
        stringRedisTemplate.expireAt(key, DateUtil.localDateTime2Date(LocalDateTime.now().plus(2L, ChronoUnit.HOURS)));

        Map<String, EvalGroupUserShop> rewardObjectBOMap = new HashMap<>();
        Map<String, BigDecimal> referCommonMap = new HashMap<>();
        for (EvalGroupUserShop rewardObject : userShops) {
            BigDecimal commissionValue = calculateParamValue(rewardObject, params);
            zSetOps.add(rewardObject.getReferId().toString(), commissionValue.doubleValue());

            referCommonMap.put(rewardObject.getReferId().toString(), commissionValue);
            rewardObjectBOMap.put(rewardObject.getReferId().toString(), rewardObject);
        }

        final BigDecimal personCount = BigDecimal.valueOf(userShops.size());
        List<EvalGroupRewardLadders> ladders = queryRewardLadders(evalGroupRewardId);
        for (EvalGroupRewardLadders ladder : ladders) {
            final BigDecimal money = ladder.getMoney();
            Integer startIndex = null;
            Integer endIndex = null;

            if (RankTypeEnum.PERCENT.equals(rankType)) {
                startIndex = ladder.getLower().multiply(personCount).divide(BigDecimal.ONE, 0, RoundingMode.FLOOR).intValue();
                endIndex = ladder.getUpper().multiply(personCount).divide(BigDecimal.ONE, 0, RoundingMode.FLOOR).intValue() - 1;
            } else {
                startIndex = ladder.getLower().intValue() - 1;
                endIndex = ladder.getUpper().intValue()- 1;
            }

            if (startIndex > endIndex) {
                continue;
            }

            Set<String> referIdStrs = zSetOps.reverseRange(startIndex, endIndex);
            for (String referIdStr : referIdStrs) {
                EvalGroupUserShop userShop = rewardObjectBOMap.get(referIdStr);
                boolean examined = examinePrecondition(userShop, preconditions);
                if (!examined) {
                    userShop.setEvalGroupRewardAmount(BigDecimal.ZERO);
                    continue;
                }
                userShop.setEvalGroupRewardAmount(money);
                BigDecimal hitCommissionValue = referCommonMap.get(referIdStr);
                clearProjectHitLog(evalGroupRewardId, userShop);
                saveProjectHitLog(evalGroupRewardId, userShop, log -> {
                    log.setRewardValue(money);
                    log.setEvalGroupRewardLaddersId(ladder.getId());
                    log.setHitLadderValue(hitCommissionValue);
                    log.setHitCommissionValue(hitCommissionValue);
                });
            }
        }

    }


}
